#include "customgui_meindatatype.h"

#include "maxon/datadescription.h"

#include "lib_description.h"
#include "c4d_customdatatypeplugin.h"


#include "maxon/datadescription_data.h"
#include "maxon/datadescription_ui.h"
#include "maxon/desctranslation.h"
#include "c4d_basedocument.h"
#include "c4d_colors.h"

#include "customgui_helpers.h"

namespace ace
{

#define MEINDATATYPEGUI_RESOURCE_SYMBOL "MEINDATATYPEGUI"
static Int32 g_resourceDataTypeTable[] = { MEINDATATYPE_ID };

Int32 MeinDataTypeData::GetId()
{
	return MEINDATATYPEGUI_ID;
}
CDialog* MeinDataTypeData::Alloc(const BaseContainer &settings)
{
	MeinDataTypeGui *dlg = NewObjClear(MeinDataTypeGui, settings, GetPlugin());
	if (!dlg)
		return nullptr;

	CDialog *cdlg = dlg->Get();
	if (!cdlg)
		return nullptr;

	return cdlg;
}
void MeinDataTypeData::Free(CDialog *dlg, void *userdata)
{
	if (!dlg || !userdata)
		return;
	MeinDataTypeGui *sub = (MeinDataTypeGui*)userdata;
	DeleteObj(sub);
}
const Char* MeinDataTypeData::GetResourceSym()
{
	return MEINDATATYPEGUI_RESOURCE_SYMBOL;
}
CustomProperty* MeinDataTypeData::GetProperties()
{
	return nullptr;
}
Int32 MeinDataTypeData::GetResourceDataType(Int32 *&table)
{
	table = g_resourceDataTypeTable;
	return sizeof(g_resourceDataTypeTable) / sizeof(Int32);
}

static MeinDataTypeLib g_meinDataTypeGuiLib;
maxon::Result<void> MeinDataTypeData::RegisterGuiPlugin()
{
	iferr_scope;

	ClearMem(&g_meinDataTypeGuiLib, sizeof(g_meinDataTypeGuiLib));
	FillBaseCustomGui(g_meinDataTypeGuiLib);

	const Bool isLibraryInstalled = InstallLibrary(MEINDATATYPEGUI_ID, &g_meinDataTypeGuiLib, 1000, sizeof(g_meinDataTypeGuiLib));
	CheckState(isLibraryInstalled == true);

	const Bool isGuiRegistered = RegisterCustomGuiPlugin("MeinDataTypeGui"_s, CUSTOMGUI_SUPPORT_LAYOUTDATA | CUSTOMGUI_DISALLOW_TAKESOVERRIDE, NewObjClear(MeinDataTypeData));
	CheckState(isGuiRegistered == true);

	return maxon::OK;
}

MeinDataTypeGui::MeinDataTypeGui(const BaseContainer &settings, CUSTOMGUIPLUGIN *plugin) : iBaseCustomGui(settings, plugin)
{
	const GeData& data = settings.GetData(DESC_DEFAULT);
	if (data.GetType() == MEINDATATYPE_ID)
	{
		_defaultValue = *static_cast<const MeinDataType*>(data.GetCustomDataType(MEINDATATYPE_ID));
	}
}

#define NUMBERFIELD_ID 1000
Bool MeinDataTypeGui::CreateLayout()
{
	static Int32 EDITW = 100;
	static Int32 EDITH = 10;

	GroupBegin(0, BFH_LEFT | BFV_CENTER, 0, 1, String(), 0);
	C4DGadget* field = AddEditNumberArrows(NUMBERFIELD_ID, 0, EDITW, EDITH);
	SetDefaultColor(field, COLOR_BGEDIT, Vector(0, 1, 1));
	GroupEnd();

	return iCustomGui::CreateLayout();
}

Bool MeinDataTypeGui::InitValues()
{
	const Float floatValue = _value.GetValue();
	SetFloat(NUMBERFIELD_ID, floatValue);
	return true;
}

Bool MeinDataTypeGui::Command(Int32 id, const BaseContainer &msg)
{
	Bool handled = true;
	switch (id)
	{
		case NUMBERFIELD_ID:
		{
			if (msg.GetBool(BFM_ACTION_RESET) == true)
			{
				_value = _defaultValue;
			}
			else
			{
				const Float floatValue = msg.GetFloat(BFM_ACTION_VALUE);
				_value = MeinDataType(floatValue);
			}
			break;
		}
		default:
		{
			handled = false;
			break;
		}
	}

	if (handled)
	{
		BaseContainer m(msg);
		m.SetInt32(BFM_ACTION_ID, GetId());
		m.SetData(BFM_ACTION_VALUE, GeData(MEINDATATYPE_ID, (const CustomDataType&)_value));
		SendParentMessage(m);
	}

	return iCustomGui::Command(id, msg);
}

TriState<GeData> MeinDataTypeGui::GetData()
{
	TriState<GeData> triState;
	triState.Add(GeData(MEINDATATYPE_ID, (const CustomDataType&)_value));
	return triState;
}

Bool MeinDataTypeGui::SetData(const TriState<GeData> &tristate)
{
	const GeData& data = tristate.GetValue();
	if (data.GetType() != MEINDATATYPE_ID)
		return false;

	const MeinDataType& value = *static_cast<const MeinDataType*>(tristate.GetValue().GetCustomDataType(MEINDATATYPE_ID));
	_value = value;
	InitValues();
	return true;
}

maxon::Result<void> MeinDataTypeGuiConversion::Initialize()
{
	iferr_scope;

	// Initialization of the default conversions on startup.
	// Here one can define which ui should appear for which data type if nothing is declared in the description.
	maxon::UiConversionInterface::AddDefaultConversion(maxon::GetDataType<ace::MeinDataType>(), MeinDataTypeGuiConversion::GetClass().GetId()) iferr_return;

	return maxon::OK;
}

maxon::Result<void> MeinDataTypeGuiConversion::QuerySupportedDataTypes(maxon::BaseArray<maxon::DataType>& dataTypes) const
{
	iferr_scope;
	dataTypes.Append(maxon::GetDataType<ace::MeinDataType>()) iferr_return;

	return maxon::OK;
}

maxon::Result<void> MeinDataTypeGuiConversion::CreateC4DDescription(const maxon::DataType& dataType, Description& c4dDescription, const maxon::LanguageRef& language,
	const maxon::DataDictionary& dataEntry, const maxon::DataDictionary& guiEntry, const maxon::DataDescription& mainDataDescription, const maxon::DataDescription& stringDescription, const DescID& mainId,
	const DescID& groupId, const maxon::PatchC4DDescriptionEntryDelegate& patchEntryFunc, maxon::DescTranslation& translateIds,
	const maxon::BaseArray<maxon::InternedId>& parentIds, const DescID& parentFoldId, const maxon::GetDataCallbackType& getDataCallback,
	const maxon::GetExtraDataCallbackType& getExtraDataDelegate) const
{
	iferr_scope;

	static const Int dtype = MEINDATATYPE_ID;
	DescID descId(mainId);
	descId.PushId(DescLevel(1, dtype, 0));

	BaseContainer param = GetCustomDataTypeDefault(dtype);
	patchEntryFunc(param) iferr_return;

	MeinDataType defaultData = dataEntry.Get(maxon::DESCRIPTION::DATA::BASE::DEFAULTVALUE.Get(), MeinDataType());
	GeData value;
	ConvertToC4D(value, dataType, maxon::Data(defaultData), DescID(), dataEntry, guiEntry, maxon::GetExtraDataCallbackType()) iferr_return;
	param.SetData(DESC_DEFAULT, value);
	// We do not have to set DESC_CUSTOMGUI explicitly, because set up in MeinDataTypeClass::GetDefaultProperties(). 

	// We have set up our custom gui and embed it into the Attribute Manager for nodes.
	CustomGuiHelpers::SetPDescription(c4dDescription, translateIds, descId, param, groupId, dataEntry, guiEntry, parentIds) iferr_return;

	return maxon::OK;
}

maxon::Result<void> MeinDataTypeGuiConversion::ConvertToC4D(GeData& output, const maxon::DataType& dataType, const maxon::Data& data, const DescID& descIdSuffix,
	const maxon::DataDictionary& dataEntry, const maxon::DataDictionary& guiEntry, const maxon::GetExtraDataCallbackType& extraDataDelegate) const
{
	iferr_scope;

	// We transport the data type from 'new' to 'classical', i.e. from node to gui.
	MeinDataType value = data.Get<MeinDataType>() iferr_return;
	output = GeData(MEINDATATYPE_ID, (CustomDataType&)value);
	return maxon::OK;
}

maxon::Result<maxon::Tuple<maxon::Data, maxon::Bool>> MeinDataTypeGuiConversion::ConvertToCore(const maxon::DataType& dataType, const GeData& data, const DescID& descIdSuffix,
	const maxon::DataDictionary& dataEntry, const maxon::DataDictionary& guiEntry, const maxon::Data& oldData,
	const maxon::GetExtraDataCallbackType& extraDataDelegate) const
{
	// We transport the data type from 'classical' to 'new', i.e. from gui to node.
	const MeinDataType& value = *static_cast<const MeinDataType*>(data.GetCustomDataType(MEINDATATYPE_ID));
	return maxon::Tuple<maxon::Data, maxon::Bool>(maxon::Data(MeinDataType(value)), false);
}

MAXON_COMPONENT_OBJECT_REGISTER(MeinDataTypeGuiConversion, maxon::UiConversions, "net.acestudio.ui.meindatatypeguiconversion");


// MAXON_DECLARATION_REGISTER(maxon::DataTypeBuilderRegistry, "net.maxonsdk.datatype.meindatatype")
// {
// 	// Show this string in the GUI.
// 	return "maxonsdk::MeinDataType"_cs;
// }

} // namespace ace